@@ -28,6 +28,7 @@ def read_csv_from_url(url,verbose=False,read_csv_kwds={}):
 
 
 def load_heroes_info(verbose=False,read_csv_kwds={}):
+
     url = 'https://raw.githubusercontent.com/jirvingphd/dsc-data-cleaning-project-online-ds-ft-100719/master/heroes_information.csv'
     return  read_csv_from_url(url, verbose=verbose,read_csv_kwds=read_csv_kwds)
     
@@ -107,6 +108,29 @@ def load_iris(verbose=False):
     return df
 
 
+def load_ames_train(verbose=False,read_csv_kwds={}):
+    """Loads height vs weight dataset"""
+    import requests 
+    if verbose:
+        res = requests.get('https://raw.githubusercontent.com/learn-co-students/dsc-project-eda-with-pandas-onl01-dtsc-pt-041320/master/data_description.txt')
+        info = res.text
+        print(info)
+    url='https://raw.githubusercontent.com/learn-co-students/dsc-project-eda-with-pandas-onl01-dtsc-pt-041320/master/ames_train.csv'
+    return  read_csv_from_url(url, verbose=verbose,read_csv_kwds=read_csv_kwds)
+
+
+def load_ames_test(verbose=False,read_csv_kwds={}):
+    import requests 
+    if verbose:
+        res = requests.get('https://raw.githubusercontent.com/learn-co-students/dsc-project-eda-with-pandas-onl01-dtsc-pt-041320/master/data_description.txt')
+        info = res.text
+        print(info)
+    """Loads height vs weight dataset"""
+    url='https://raw.githubusercontent.com/learn-co-students/dsc-project-eda-with-pandas-onl01-dtsc-pt-041320/master/ames_test.csv'
+    return  read_csv_from_url(url, verbose=verbose,read_csv_kwds=read_csv_kwds)
+
+
+
 def load_height_weight(verbose=False,read_csv_kwds={}):
     """Loads height vs weight dataset"""
     url='https://raw.githubusercontent.com/jirvingphd/dsc-probability-density-function-online-ds-ft-100719/master/weight-height.csv'
@@ -241,3 +265,22 @@ def load_stock_df(**kwargs):
     stock_df = pd.read_excel(url,**kwargs)
     return stock_df
     
+
+def load_heart_disease(verbose=False,read_csv_kwds={}):
+    import pandas as pd
+    url = "https://raw.githubusercontent.com/jirvingphd/dsc-gaussian-naive-bayes-lab-online-ds-pt-100719/solution/heart.csv"
+    return read_csv_from_url(url, verbose=verbose,read_csv_kwds=read_csv_kwds)
+
+
+def load_spam(verbose=False,read_csv_kwds=dict( sep='\t', names=['label', 'text'])):
+    url = "https://raw.githubusercontent.com/jirvingphd/dsc-document-classification-with-naive-bayes-lab-online-ds-pt-100719/solution/SMSSpamCollection"
+    return read_csv_from_url(url, verbose=verbose,read_csv_kwds=read_csv_kwds)
+
+
+
+
+def load_tennis(read_csv_kwds={}):
+    url="https://raw.githubusercontent.com/jirvingphd/dsc-decision-trees-with-sklearn-codealong-online-ds-pt-100719/master/tennis.csv"
+    return read_csv_from_url(url,verbose=False, read_csv_kwds=read_csv_kwds)
+
+
