@@ -4,8 +4,8 @@ Email: james.irving.phd@gmail.com
 GitHub Profile: https://github.com/jirvingphd
 """
 # import fsds_100719 as fs 
-from fsds_100719.ds import ihelp,ihelp_menu
-from fsds_100719.jmi import reload, print_docstring_template
+from fsds_100719.ds import ihelp,ihelp_menu, reload
+# from fsds_100719.jmi import print_docstring_template
 # print(f"[i] You're using V {fs.__version__} of fsds.")
 
 def testing():
